"""
Rich is a Python library for rich text and beautiful formatting in the terminal.
This script constructs rich console object.
"""

from rich.console import Console

console = Console(color_system="truecolor", highlight=False)
